#include <cmath>
#include "resummationQCD.hh"
#include "chebyshev.hh"
#include "QCD_EW.hh"
#include "math/special_functions.hh"
#include "math/integration.hh"
#include "math/struct_definitions.hh"
#include <gsl/gsl_sf.h>
#include <gsl/gsl_errno.h>
#include <cuba.h>

#include "../../ggHiggs/src/NLO_finite-mt/bdv.hh"
#include "../../ggHiggs/src/NNLO_finite-mt/nnlo_finite-mt.hh"

#include "LHAPDF/LHAPDF.h"


using namespace std;




namespace resumm {


  QCDResummation::~QCDResummation() {
    return;
  }


  double QCDResummation::res_xs(double m, double sqrt_s,
				double muF_m_ratio, double muR_m_ratio,
				log_order LOGo, order match_order, double *asinput) {
    doRapidityDistr = false;
    //
    tau = m*m/sqrt_s/sqrt_s;
    M   = m;
    muF = m*muF_m_ratio;
    muR = m*muR_m_ratio;
    //
    LL_order = LOGo;
    matching_order = match_order;
    //
    if(asinput == NULL) as = alphas(muR); // /50.;
    else as = *asinput; // /50.;
    //cout << as << endl;
    //
    bool redo_cheb = !(last_M==M && last_muFrat==muF_m_ratio && last_sqrts==sqrt_s && last_Ncheb==Nchebyshev);
    ComputeCommon(redo_cheb);
    Chebyshev cheb(this);
    //
    //gsl_error_handler_t *old_handler = gsl_set_error_handler (NULL);
    //gsl_set_error_handler_off();
    //
    double result = 0.;

    if(prescr < 0 || expBP) { // then it's a MP
      MP_slope = 1.;
      MP_reN0 = MPbeta + 3.;
      double NL = Nlandau();
      if(MP_reN0 >= NL) {
	MP_reN0 = ( MPbeta + NL )/2.;
	if(MP_reN0 >= NL) {
	  cout << "ERROR: Landau pole N_L = " << NL
	       << " to the left of the rightmost singularity: " << MPbeta << endl;
	  exit(-13);
	}
      }
      MPzmin=tau;
      //if(tau<MPzmin) MPzmin=tau;
      if(redo_cheb) cheb.ApproxLuminosity(coeff, Nchebyshev, MPbeta, MPgamma, MPzmin);
      result = MinimalPrescription()/luminosity_in_tau;
    }
    else if(prescr == BP0 || prescr == BP00) {
      if(redo_cheb) cheb.ApproxLumPlus_log1z(coeff, tau, Nchebyshev);
      result = BorelPrescription();
    }
    else {
      if(redo_cheb) cheb.ApproxLumPlus(coeff, tau, Nchebyshev);
      result = BorelPrescription();
    }
    //gsl_set_error_handler (old_handler);
    //
    last_muFrat = muF_m_ratio;
    last_muRrat = muR_m_ratio;
    last_M      = M;
    last_sqrts  = sqrt_s;
    last_nf     = nf;
    last_mtop   = mtop;
    last_EFT    = mtEFT;
    last_Ncheb  = Nchebyshev;
    firstinit = false;
    //
    return result;
  }


  double QCDResummation::res_xs_rap(double m, double Y, double sqrt_s,
				    double muF_m_ratio, double muR_m_ratio,
				    log_order LOGo, order match_order, double *asinput) {
    doRapidityDistr = true;
    //
    rapY = Y;
    tau = m*m/sqrt_s/sqrt_s;
    M   = m;
    muF = m*muF_m_ratio;
    muR = m*muR_m_ratio;
    //
    LL_order = LOGo;
    matching_order = match_order;
    //
    if(asinput == NULL) as = alphas(muR); // /50.;
    else as = *asinput; // /50.;
    //cout << as << endl;
    //
    bool redo_cheb = !(last_M==M && last_muFrat==muF_m_ratio && last_sqrts==sqrt_s && last_Ncheb==Nchebyshev);
    ComputeCommon(redo_cheb);
    Chebyshev cheb(this);
    //
    //gsl_error_handler_t *old_handler = gsl_set_error_handler (NULL);
    //gsl_set_error_handler_off();
    //
    double result = 0.;

    if(prescr < 0 || expBP) { // then it's a MP
      MP_slope = 1.;
      MP_reN0 = MPbeta + 3.;
      double NL = Nlandau();
      if(MP_reN0 >= NL) {
	MP_reN0 = ( MPbeta + NL )/2.;
	if(MP_reN0 >= NL) {
	  cout << "ERROR: Landau pole N_L = " << NL
	       << " to the left of the rightmost singularity: " << MPbeta << endl;
	  exit(-13);
	}
      }
      MPzmin=tau;
      //if(tau<MPzmin) MPzmin=tau;
      if(redo_cheb) cheb.ApproxLuminosity(coeff, Nchebyshev, MPbeta, MPgamma, MPzmin);
      result = MinimalPrescription()/luminosity_in_tau;
    }
    else if(prescr == BP0 || prescr == BP00) {
      if(redo_cheb) cheb.ApproxLumPlus_log1z(coeff, tau, Nchebyshev);
      result = BorelPrescription();
    }
    else {
      if(redo_cheb) cheb.ApproxLumPlus(coeff, tau, Nchebyshev);
      result = BorelPrescription();
    }
    //gsl_set_error_handler (old_handler);
    //
    last_muFrat = muF_m_ratio;
    last_muRrat = muR_m_ratio;
    last_M      = M;
    last_sqrts  = sqrt_s;
    last_nf     = nf;
    last_mtop   = mtop;
    last_EFT    = mtEFT;
    last_Ncheb  = Nchebyshev;
    firstinit = false;
    //
    return result;
  }





};






